@@ -32,8 +32,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define RANDOMCOIN_INTENSITY (COMMON_GAUSS + 1) 
 
 
-#define PGS_RATIO 0
-#define PGS_FLAT 1
 
 #define PGS_VOXEL 0
 #define PGS_CORNER 1
@@ -58,55 +56,22 @@ int addUnifModel(cov_model VARIABLE_IS_NOT_USED *cov, double radius, cov_model *
 }
 
 
-int alloc_pgs(cov_model *cov) {
-  return alloc_pgs(cov, cov->xdimown);
-}
-
-int alloc_pgs(cov_model *cov, int dim) { // all what is necessary for dompp
-  pgs_storage *pgs = NULL;
-
-  if (cov->Spgs != NULL) PGS_DELETE(&(cov->Spgs));
-  if ((pgs = cov->Spgs = (pgs_storage*) MALLOC(sizeof(pgs_storage))) == NULL) 
-    return ERRORMEMORYALLOCATION;
-  PGS_NULL(pgs);
-
-  //printf("here\n");
-  
-  if ((pgs->supportmin = (double*) CALLOC(dim, sizeof(double))) == NULL ||
-      (pgs->supportmax = (double*) CALLOC(dim, sizeof(double))) == NULL ||
-      (pgs->supportcentre = (double*) CALLOC(dim, sizeof(double))) == NULL || 
-      
-      (pgs->gridlen = (int*) CALLOC(dim, sizeof(int))) == NULL ||
-      (pgs->end = (int*) CALLOC(dim, sizeof(int))) == NULL ||
-      (pgs->start = (int*) CALLOC(dim, sizeof(int))) == NULL ||
-      (pgs->delta = (int*) CALLOC(dim, sizeof(int))) == NULL ||
-      (pgs->nx = (int*) CALLOC(dim, sizeof(int))) == NULL ||
-   
-      (pgs->xstart = (double*) CALLOC(dim, sizeof(double))) == NULL || 
-      (pgs->x = (double*) CALLOC(dim, sizeof(double))) == NULL ||
-      (pgs->inc = (double*) CALLOC(dim, sizeof(double))) == NULL)
-    return ERRORMEMORYALLOCATION;
-
-  // printf("here end\n");
-  return NOERROR;
-}
-
 
 
 
 void pts_given_shape(double *x, cov_model *cov, double *v) { 
   cov_model *shape = cov->sub[PGS_FCT];
   NONSTATCOV(x, cov->q, shape, v);
-  //  if (cov->q[0] < 1) {
-  //   printf("x=%f q=%f v=%f\n", x[0], cov->q[0], v[0]);
-  //   assert(false);
-  // } 
-  //  APMI(cov);
+
+  //printf("x = %f %f %v\n", *x, cov->q[0], *v);
+
 }
 
 void logpts_given_shape(double *x, cov_model *cov, double *v, double *sign) { 
   cov_model *shape = cov->sub[PGS_FCT];
   LOGNONSTATCOV(x, cov->q, shape, v, sign);
+
+  //  printf("x = %f %f log=%f\n", *x, cov->q[0], *v, *sign);
 }
 
 int check_pts_given_shape(cov_model *cov) {
@@ -120,6 +85,10 @@ int check_pts_given_shape(cov_model *cov) {
    
   kdefault(cov, PGS_RATIO, GLOBAL.extreme.density_ratio); 
   kdefault(cov, PGS_FLAT, GLOBAL.extreme.flat);
+  kdefault(cov, PGS_INFTY_SMALL, !PINT(PGS_FLAT));
+  kdefault(cov, PGS_NORMED, true);
+  kdefault(cov, PGS_ISOTROPIC, true);
+
 
   //PMI(cov, "checking");
  
@@ -148,14 +117,27 @@ int check_pts_given_shape(cov_model *cov) {
   } else ILLEGAL_ROLE;
 
 
-  //printf("here\n");
+  //    printf("here %d\n");
   
-  // isotropy is the property
-  if ((err = CHECK(shape, dim, dim, ShapeType, XONLY, ISOTROPIC,
-		     SCALAR, role)) != NOERROR)  return err;
-  // but it must be treated as if it was non-isotropic
+  // isotropy is the property -- user should not define unisotropic functions!
+  // so !isotropic only for special, internal definitions, e.g. 
+  // strokorbPoly
+  // int err2 = NOERROR;
+  //  if (P0INT(PGS_ISOTROPIC)) 
+  //    err2 = CHECK(shape, dim, dim, ShapeType, XONLY, ISOTROPIC,
+  //		 SCALAR, role);
+
+  //  printf("here2\n");
+  // PMI(shape);
+ // but it must be treated in any case as if it was non-isotropic
   if ((err = CHECK(shape, dim, dim, ShapeType, XONLY, CARTESIAN_COORD,
-		     SCALAR, role)) != NOERROR)  BUG;
+		   SCALAR, role)) != NOERROR) {
+    if (P0INT(PGS_ISOTROPIC)) BUG;
+    XERR(err);
+    return err;
+  }
+  // if (err2 != NOERROR) SERR("isotropic shape function expected.");
+
   setbackward(cov, shape);
 
   //if (!shape->deterministic) return ERRORNOTPROGRAMMED; // Dichte muss nicht normiert sein und stattdessen durch das mittlere Volumen dividiert werden.
@@ -165,6 +147,10 @@ int check_pts_given_shape(cov_model *cov) {
     //printf("pts %s %s\n", CovList[pts->nr + 1].nick, CovList[pts->nr + 1].name);
     //PMI(pts);
 
+    //      assert(dim == 2); print("x");
+    // printf("dim=%d\n", dim);
+    // APMI(pts);
+
     if ((err = CHECK_R(pts, dim)) != NOERROR) return err;
   }
 
@@ -181,7 +167,7 @@ int struct_pts_given_shape(cov_model *cov, cov_model **newmodel){
   int err = NOERROR;
 
   if (newmodel != NULL) BUG;
-  if (cov->Spgs != NULL) free(cov->Spgs);
+  if (cov->Spgs != NULL)  PGS_DELETE(&(cov->Spgs));
 
   if (shape->role != ROLE_POISSON && shape->role != ROLE_MAXSTABLE)
     ILLEGAL_ROLE;
@@ -264,7 +250,7 @@ int calculate_mass_gauss(cov_model *cov) {
     COV(ZERO, shape, v);
     v[0] *= intpow(0.5, dim);
     NONSTATINVERSE_D(v, shape, x, y); 
-    if (ISNA(x[0]) || x[0] > y[0]) 
+    if (ISNAN(x[0]) || x[0] > y[0]) 
       SERR1("inverse function of '%s' unknown", NICK(shape));
 
   
@@ -285,7 +271,7 @@ int calculate_mass_gauss(cov_model *cov) {
     }
     if (zx != v[0] || zy != v[0] ||  true) {
       for (d=0; d<dim; d++) {
-	y[d] /= sqrt((double)dim); // falls nicht runif
+	y[d] /= sqrt((double) dim); // falls nicht runif
       }
     }
     
@@ -299,7 +285,7 @@ int calculate_mass_gauss(cov_model *cov) {
 	double range = loc->xgr[d][XSTEP] * (loc->xgr[d][XLENGTH] - 1.0);
 	xgr[d][XLENGTH] = ceil(range / y[d] + 1.0);
 	if (xgr[d][XLENGTH] >= loc->xgr[d][XLENGTH]) {
-	  BUG; assert(false);
+	  BUG;
 	  memcpy(xgr[d], loc->xgr[d], 3 * sizeof(double));
 	} else {
 	  xgr[d][XSTART] =
@@ -330,18 +316,20 @@ int calculate_mass_gauss(cov_model *cov) {
 }
 
 int calculate_mass_maxstable(cov_model *cov) {
+
   pgs_storage *pgs = cov->Spgs;
   location_type *loc = Loc(cov);
   cov_model *shape = cov->sub[PGS_FCT],
     *pts = cov->sub[PGS_LOC];
-  double voxels,
+  double voxels, value_orig,
     *single = pgs->single, // out
     *total = pgs->total,   // out
     *halfstepvector = pgs->halfstepvector, //out
     *x = pgs->x;
   //  bool flat = false;
   int d,
-    dim = cov->tsdim;
+    dim = cov->tsdim,
+    flat =P0INT(PGS_FLAT);
  
   if (shape->role == ROLE_POISSON) {
     BUG;
@@ -350,28 +338,28 @@ int calculate_mass_maxstable(cov_model *cov) {
 
   } // else {
 
-  for (d=0; d<dim; d++) halfstepvector[d] = 0.0; 
-
-  //  printf("hier calculate_mass\n");
-  VTLG_D(halfstepvector, pts, &(pgs->value_orig));
+  assert(dim <= MAXSIMUDIM);
+  VTLG_D(ZERO, pts, &value_orig);
 
   //  PMI(cov, "calculate max");
+  //  printf("%s %d %d %f\n",NICK(cov), PGS_FLAT, P0INT(PGS_FLAT), value_orig );// APMI(cov); assert(false);
   
-  int flat =P0INT(PGS_FLAT);
-  //  printf("%s %d %d %f\n",NICK(cov), PGS_FLAT, ((int*)cov->p[PGS_FLAT])[0], pgs->value_orig );// APMI(cov); assert(false);
-    
+  // VOXEL 
+  for (d=0; d<dim; d++) halfstepvector[d] = 0.5 * loc->xgr[d][XSTEP];     
   if (flat == FLAT_UNDETERMINED) {
     // flat == im Kerngebiet wird konstant simuliert; ausserhalb dann
     // abfallend
     if (loc->grid) {
       double v;
-      for (d=0; d<dim; d++) halfstepvector[d] = 0.5 * loc->xgr[d][XSTEP];     
       VTLG_D(halfstepvector, pts, &v);
       double threshold = 
-	pgs->value_orig == RF_INF//&& cov->p[PGS_RATIO][0] == 0.0 
+	value_orig == RF_INF//&& cov->p[PGS_RATIO][0] == 0.0 
 	? RF_INF
-	: pgs->value_orig * P0(PGS_RATIO);
-      pgs->flat =  threshold < v;
+	: value_orig * P0(PGS_RATIO);
+      pgs->flat =  threshold < v && cov->deterministic; // sicherheitshalber
+
+      //printf("thres %f %f %f\n", threshold, v, value_orig);
+
     } else { // not grid
       BUG;
       // long term project: to do
@@ -386,22 +374,34 @@ int calculate_mass_maxstable(cov_model *cov) {
   } else pgs->flat = flat;
 
   if (pgs->flat) {
-    single[PGS_VOXEL] = pgs->value_orig;
-    for (d=0; d<dim; d++) single[PGS_VOXEL] *= loc->xgr[d][XSTEP];
+    if (P0INT(PGS_INFTY_SMALL)) {
+      //PMI(cov);
+      SERR2("'%s' and '%s' may not be positive at the same time",
+	    KNAME(PGS_FLAT), KNAME(PGS_INFTY_SMALL));
+    }
+    single[PGS_VOXEL] = value_orig;
+    for (d=0; d<dim; d++) single[PGS_VOXEL] *= 2.0 * halfstepvector[d];
   } else {
     //    APMI(cov);
     //       printf("two sided %f %f %f\n", halfstepvector[0], halfstepvector[1], halfstepvector[2]); //assert(false);
     VTLG_P2SIDED(NULL, halfstepvector, pts, single + PGS_VOXEL);
-    //   printf("two sided done %f %s\n", single[PGS_VOXEL], NICK(pts)); assert(false);
+    //     printf("two sided done %e %f %s\n", single[PGS_VOXEL], halfstepvector[0], NICK(pts)); //assert(false);
   }
   voxels = 1.0;
   for (d=0; d<dim; d++) voxels *= loc->xgr[d][XLENGTH] - 1.0;  
   total[PGS_VOXEL] = single[PGS_VOXEL] * voxels;
   
+  // CORNER
   for (d=0; d<dim; d++) x[d] = RF_INF;  
   VTLG_P2SIDED(NULL, x, pts, single + PGS_CORNER);
-  assert(single[PGS_CORNER] == 1.0); // nur wenn normiert
-  total[PGS_CORNER] = 1.0 + total[PGS_VOXEL];
+
+  //  printf("%d %f\n", !P0INT(PGS_NORMED), single[PGS_CORNER]);
+
+  assert(!P0INT(PGS_NORMED) || single[PGS_CORNER] == 1.0); // nur wenn normiert
+  total[PGS_CORNER] = single[PGS_CORNER] + total[PGS_VOXEL];
+
+  //  printf("ier\n");
+  //  PMI(cov);
 
   if (dim >= 2) {
     for (d=0; d<dim; d++) {
@@ -444,7 +444,8 @@ int calculate_mass_maxstable(cov_model *cov) {
   }
   //	APMI(cov);
 
-  if (!R_FINITE(pgs->totalmass = total[pgs->size - 1])) {
+  pgs->totalmass = total[pgs->size - 1];
+  if (!R_FINITE(pgs->totalmass)) {
     // int i; for (i=0; i<pgs->size; i++) printf("i=%d total=%f\n", i, total[i]);
     SERR("the total intensity mass is not finite");
   }
@@ -453,6 +454,66 @@ int calculate_mass_maxstable(cov_model *cov) {
 }
 
 
+cov_model *prunecov(cov_model *newmodel, cov_model *cov) {
+  //printf("names= %s %s\n", NICK(newmodel->calling), NICK(cov->calling));
+  cov_model
+    **sub = NULL,
+    *next = NULL,
+    *calling = cov->calling;
+  if (calling == newmodel->calling) {
+    return newmodel;
+  }
+  if (calling == NULL) BUG;
+  prunecov(newmodel, calling);
+  if (calling->key == cov) sub = &(newmodel->key);
+  else if (calling->sub[0] == cov) sub = newmodel->sub + 0;
+  else if (calling->sub[1] == cov) sub = newmodel->sub + 1;
+  else BUG;
+  next = *sub;
+  *sub = NULL;
+  COV_DELETE(&newmodel);
+  return next;
+}
+
+int complete_copy(cov_model **newmodel, cov_model *cov) {
+  cov_model *prev = NULL,
+    *start = cov;
+  int err;
+  while (start->calling != NULL) start = start->calling;
+  if (start->typus != InterfaceType) BUG; // nur zur Sicherheit, derzeit; kann erweitert werden
+  if (start == cov) BUG;
+  prev = start->key != NULL ? start->key : start->sub[0];
+  if (prev->typus != ProcessType) BUG;
+  
+  if ((err = covcpy(newmodel, prev)) != NOERROR) return err;
+  (*newmodel)->calling = cov;
+  int role = prev->role ;// role_of_process(prev->nr);
+  if ((err = CHECK(*newmodel, prev->tsdim, prev->xdimprev, prev->typus, 
+		   prev->domprev, prev->isoprev, prev->vdim2, role)) 
+      != NOERROR)  return err;  
+   if ((err = STRUCT(*newmodel, NULL)) != NOERROR) { return err; }
+  if (!(*newmodel)->initialised) {
+    if ((err =  CHECK(*newmodel, prev->tsdim, prev->xdimprev, prev->typus, 
+		      prev->domprev, prev->isoprev, prev->vdim2, role)) 
+	!= NOERROR) return err;  
+ 
+    assert((*newmodel)->stor == NULL);
+    (*newmodel)->stor = (storage *) MALLOC(sizeof(storage)); 
+    STORAGE_NULL((*newmodel)->stor);
+    
+    //APMI(*newmodel);
+    if ((err = INIT(*newmodel, 0, (*newmodel)->stor)) != NOERROR) {
+      //APMI(cov); // !!! ?? hier weitermachen
+      return err; 
+    }
+  }
+ 
+  ///APMI(cov);
+  (*newmodel)->calling = start;
+  *newmodel = prunecov(*newmodel, cov);
+  (*newmodel)->calling = NULL;
+  return NOERROR;
+}
 
 int init_pts_given_shape(cov_model *cov, storage *S) {  
   cov_model *shape = cov->sub[PGS_FCT],
@@ -462,8 +523,10 @@ int init_pts_given_shape(cov_model *cov, storage *S) {
   int d, i,
     err = NOERROR,
     dim = shape->xdimprev;
-  bool grid = Loc(cov)->grid;
-
+  pgs_storage *pgs = cov->Spgs;
+  bool 
+    grid = Loc(cov)->grid,
+    pgsnull = pgs == NULL;
   //PMI(cov);
 
   assert(cov->sub[PGS_LOC] != NULL);
@@ -474,13 +537,18 @@ int init_pts_given_shape(cov_model *cov, storage *S) {
     SERR1("support of the model is unknown. Use '%s' to determine the support",
 	  CovList[TRUNCSUPPORT].nick);
  
-  if ((err = alloc_pgs(cov)) != NOERROR) return err;
-  pgs_storage *pgs = cov->Spgs;
-
-  if ((pgs->v = (double*) CALLOC(dim, sizeof(double))) == NULL ||
-      (pgs->y = (double*) CALLOC(dim, sizeof(double))) == NULL
+  if (pgsnull) {
+    if ((err = alloc_pgs(cov)) != NOERROR) return err;
+    pgs = cov->Spgs;
+    
+    if ((pgs->v = (double*) CALLOC(dim, sizeof(double))) == NULL ||
+	(pgs->y = (double*) CALLOC(dim, sizeof(double))) == NULL
       ) return ERRORMEMORYALLOCATION;
-  
+
+    pgs->zhou_c = 1.0;
+    pgs->sum_zhou_c = pgs->sq_zhou_c = 0.0;
+    pgs->n_zhou_c = 0;
+  }
   
   // PMI(cov, "hi");
   assert(pts == cov->sub[PGS_LOC]);
@@ -499,10 +567,11 @@ int init_pts_given_shape(cov_model *cov, storage *S) {
   assert(shape->mpp.moments >= 1);
 
   //  printf("AB moments=%d\n", cov->mpp.moments);
-  if ((err = INIT(pts, 1, S)) != NOERROR) {
+  //APMI(pts->calling);
+  if ((err = INIT(pts, 0, S)) != NOERROR) { // , 0, S statt ,1,S
     return err; 
   }
-  //   printf("AC moments=%d\n", cov->mpp.moments);
+  //     printf("AC moments=%d\n", cov->mpp.moments);
 
   if (!grid) { // todo
     SERR("non-grid not programmed yet"); // meiste da; fehlt noch
@@ -511,52 +580,101 @@ int init_pts_given_shape(cov_model *cov, storage *S) {
 
   pgs->size =  grid ? intpow(2, dim) : loc->totalpoints;
   if (cov->role == ROLE_POISSON_GAUSS) {
-    if (
-	(pgs->xgr[0] = (double*) CALLOC(dim * 3, sizeof(double))) == NULL ||
-	(pgs->pos = (int *) CALLOC(dim, sizeof(int))) ==NULL ||
-	(pgs->min = (int*) CALLOC(dim, sizeof(int))) == NULL ||
-	(pgs->max = (int*) CALLOC(dim, sizeof(int))) == NULL
-	) return ERRORMEMORYALLOCATION;
+    if (pgsnull &&
+	(
+	 (pgs->xgr[0] = (double*) CALLOC(dim * 3, sizeof(double))) == NULL ||
+	 (pgs->pos = (int *) CALLOC(dim, sizeof(int))) ==NULL ||
+	 (pgs->min = (int*) CALLOC(dim, sizeof(int))) == NULL ||
+	 (pgs->max = (int*) CALLOC(dim, sizeof(int))) == NULL
+	 )) return ERRORMEMORYALLOCATION;
     for (i=3, d=1; d<dim; d++, i+=3) pgs->xgr[d] = pgs->xgr[0] + i;
     // printf("%d\n", pgs->xgr[1] - pgs->xgr[0]); assert(false);
    if ((err = calculate_mass_gauss(cov)) != NOERROR) return err;
   } else if (hasMaxStableRole(cov)) {
-    if (
+    if (pgsnull &&
+	(
 	(pgs->single = (double*) CALLOC(pgs->size, sizeof(double)))==NULL ||
 	(pgs->total = (double*) CALLOC(pgs->size, sizeof(double))) ==NULL ||
 	(pgs->halfstepvector = (double*) CALLOC(dim, sizeof(double)))==NULL
-	)
+	 ))
       return ERRORMEMORYALLOCATION;
 
     // APMI(cov);
 
-    if ((err = calculate_mass_maxstable(cov)) != NOERROR) return err;
-    cov->mpp.log_zhou_c = log(pgs->totalmass); 
+    assert(!cov->deterministic || P0INT(PGS_NORMED));
+    
+   
+    //         printf("init %f zhou=%f\n", pgs->totalmass, cov->mpp.zhou_c);
+    if (P0INT(PGS_NORMED)) {
+      if (R_FINITE(pts->mpp.unnormedmass)) { 
+	// APMI(pts);
 
-    //  printf("zhou %f %f\n",  cov->mpp.log_zhou_c, pgs->totalmass); APMI(cov); assert(false); // x,x,x:zhou 2.161274 8.682194; x,x,0: zhou 1.177634 3.246682
+	// TO DO !! Richtig nur falls Huetchen derministisch ist !!!
 
+	cov->mpp.maxheights[0] = pts->mpp.unnormedmass / shape->mpp.mMplus[1]; // korrekt, da pts * mass >= shape  !!
 
-    if (shape->deterministic) {
-      cov->mpp.maxheight = pts->mpp.maxheight * shape->mpp.maxheight;
-      if (!R_FINITE(cov->mpp.maxheight)) BUG;
-    } else { // currently both cases are identical ...
-      cov->mpp.maxheight = pts->mpp.maxheight * shape->mpp.maxheight;
-      if (!R_FINITE(cov->mpp.maxheight)) {
-	//	APMI(cov);
+	//APMI(cov);
+
+	//	
+      } else {
+	//PMI(pts->calling);
+	//	
+	// BUG; // not programmed yet
+	// to do: funktioniert nur bei ausgewaehlten funktionen. Z.B. Gauss
+	cov->mpp.maxheights[0] = shape->mpp.maxheights[0] * pts->mpp.maxheights[0];
+	//     APMI(cov);
+      }
+    } else {
+      if (R_FINITE(pts->mpp.maxheights[0])) {
+	// eigentlich ein 1.0 / pts->mpp.minheight
+	cov->mpp.maxheights[0] = pts->mpp.maxheights[0] * shape->mpp.maxheights[0];
+      } else {
 	BUG;
       }
+    }			   
+ 
+      // f_raw,rectangular must be always >= the shape function.
+    // this is the fundamental condition
+    if (!R_FINITE(cov->mpp.maxheights[0])) BUG;
+
+    if ((cov->deterministic = shape->deterministic)) {
+      if ((err = calculate_mass_maxstable(cov)) != NOERROR) return err;
+      pgs->zhou_c = pgs->totalmass / shape->mpp.mMplus[1];
+
+      //     printf("%f %f %f\n", pgs->zhou_c, pgs->totalmass, shape->mpp.mMplus[1]);assert(false);
+
+    } else { // currently both cases are identical ... 
+      pgs->zhou_c = RF_NA;
+      if (pgs->cov == NULL) {
+	cov_model *start=cov->calling;
+	if (start == NULL) BUG;	
+	//	APMI(start);
+	while (start->calling != NULL && start->nr != PTS_GIVEN_SHAPE)
+	  start = start->calling;	
+	if (start->nr != PTS_GIVEN_SHAPE) {
+	  if ((err = complete_copy(&(pgs->cov), cov)) != NOERROR) return err;
+	  pgs->cov->calling = cov->calling;
+	  pgs->cov->Spgs->cov = cov;
+	  //PMI(cov);	  APMI(pgs->cov);
+	} 
+      }
+      pgs->old_zhou = 0.0;
     }
+    pgs->estimated_zhou_c = !cov->deterministic;
 
 
+    //    PMI(cov);    assert(!pgs->estimated_zhou_c);
 
   } else {
     // APMI(cov);
     BUG;
   }
+  
 
 
   if (CovList[shape->nr].nonstat_inverse == ErrInverseNonstat) {
     if (pts->nr != RECTANGULAR) {
+      //       PMI(pts);
       // APMI(shape);
       warning("Inverse of shape function cannot be determined. Simulation speed  might be heavily decreased.");
     }
@@ -564,21 +682,20 @@ int init_pts_given_shape(cov_model *cov, storage *S) {
   
  
   for (i=0; i<=cov->mpp.moments; i++) {
-    //    printf("%d %f %f %d\n", i, shape->mpp.M[i], shape->mpp.Mplus[i], cov->mpp.moments);
+    //    printf("%d %f %f %d\n", i, pts->mpp.mM[i], shape->mpp.mMplus[i], cov->mpp.moments);
 
-    cov->mpp.M[i] = pts->mpp.M[i];
-    cov->mpp.Mplus[i] = pts->mpp.Mplus[i];
+    //assert(pts->mpp.mMplus[0] == 1.0);
+    
+    cov->mpp.mM[i] = shape->mpp.mM[i] * pts->mpp.mMplus[0];
+    cov->mpp.mMplus[i] = shape->mpp.mMplus[i] * pts->mpp.mMplus[0];
   }
+
+  
    
   cov->rf = shape->rf;
   cov->origrf = false;
  
-
-  //APMI(cov);
-
-  // assert(false);
-
-  //  printf("init done %d\n", err);
+  // APMI(cov);
 
   return err;
 }
@@ -633,23 +750,11 @@ void do_pgs_gauss(cov_model *cov, storage *S) {
 
 
   if (!cov->deterministic) {
-    int err;
     DO(shape, S); 
-    // APMI(pts);
-    if ((err = INIT(pts, 1, S)) != NOERROR) {
-      XERR(err); 
-    }
     DORANDOM(pts, cov->q);  // cov->q nur dummy. Wird ueberschrieben
     if (cov->role == ROLE_POISSON_GAUSS || !grid) { 
       if (calculate_mass_gauss(cov) != NOERROR) 
 	error("unexpected error in 'do_pts_given_shape' (maxstable)");
-    } else if (cov->role == ROLE_MAXSTABLE) { 
-      BUG;
-      if (calculate_mass_maxstable(cov) != NOERROR)
-	error("Unexpected error in 'do_pts_given_shape' (maxstable)");  ;
-      cov->mpp.log_zhou_c = log(pgs->totalmass);
-      cov->mpp.maxheight = pts->mpp.maxheight * shape->mpp.maxheight;
-      if (!R_FINITE(cov->mpp.maxheight)) BUG;
     } else BUG;
   }
 
@@ -664,7 +769,7 @@ void do_pgs_gauss(cov_model *cov, storage *S) {
   //printf("cat=%d el=%d\n", i, elmts);
   if (loc->grid) {
     NONSTATINVERSE_D(&gauss_eps, pts, x, y);
-    if (ISNA(x[0]) || x[0] > y[0]) BUG;
+    if (ISNAN(x[0]) || x[0] > y[0]) BUG;
 
     //    printf("extremes %f %f %f\n", eps, x[0], y[0]);
 
@@ -742,43 +847,138 @@ void do_pgs_gauss(cov_model *cov, storage *S) {
 }
 
 
-//static int counter[2]= {0,0};
+#define SET_COV					 \
+  pgs = cov->Spgs;				 \
+  shape = cov->sub[PGS_FCT];			 \
+  pts = cov->sub[PGS_LOC];			 \
+  loc = Loc(cov);				 \
+  single = pgs->single;				 \
+  total = pgs->total;				 \
+  halfstepvector = pgs->halfstepvector;		 \
+  x = pgs->x;					 \
+  v = pgs->v;					 \
+  dim = shape->xdimprev;			 \
+  flat = pgs->flat;				 \
+  assert(pgs != NULL);				 \
+  assert(cov->deterministic ||			 \
+	 (pgs->cov!=NULL && pgs->cov->Spgs!=NULL \
+	  && pgs->cov->Spgs->cov==cov));
 
+#define SWAP_PGS				\
+  cov = cov->Spgs->cov;				\
+  SET_COV
+
+ 
+//static double sumArea = 0.0;
 void do_pgs_maxstable(cov_model *cov, storage *S) {
-  pgs_storage *pgs = cov->Spgs;
-  cov_model *shape = cov->sub[PGS_FCT],
-    *pts = cov->sub[PGS_LOC];
-  location_type *loc = Loc(cov);
-  //window_info *w = &(S->window);
-  int i, d, elmts,
-    dim = shape->xdimprev;
+  pgs_storage *pgs = NULL;
+  cov_model *shape = NULL,
+    *pts = NULL;
+  location_type *loc = NULL;
+  int i, d, elmts, mcmc, dim, err;
   double 
-    *single = pgs->single, // in
-    *total = pgs->total,   // in
-    *halfstepvector = pgs->halfstepvector, // in
-    *x = pgs->x, // dummy
-    *v = pgs->v; // dummy
-  bool
-    flat = pgs->flat;
+    *single = NULL, // in
+    *total = NULL,   // in
+    *halfstepvector = NULL, // in
+    *x = NULL, // dummy
+    *v = NULL; // dummy
+  bool flat;
+  assert(cov->calling != NULL);
+
+  if (cov->deterministic) {
+    SET_COV;
+  } else {
+    // assert(PARAM0(shape->sub[0], POWSCALE) == PARAM0(pts, LOC_SCALE));
+    double cmaxDmu;
+    SWAP_PGS;    
+    for (mcmc=0; mcmc<GLOBAL.extreme.mcmc_zhou; mcmc++) {
+      if ((err = REINIT(cov, cov->mpp.moments, S)) != NOERROR) BUG;
+      DO(shape, S);     
+      if (calculate_mass_maxstable(cov) != NOERROR) 
+	error("unexpected error in 'do_pts_given_shape' (maxstable)");
+
+      cmaxDmu = pgs->totalmass / shape->mpp.mMplus[1];
+      if (pgs->n_zhou_c < GLOBAL.extreme.max_n_zhou) {
+	pgs->n_zhou_c++;
+	pgs->sum_zhou_c += cmaxDmu;
+	pgs->sq_zhou_c += cmaxDmu * cmaxDmu;
+	// sumArea += getArea( shape->sub[0]->Spolygon->P);
+	//if (pgs->n_zhou_c % 1000 == 0) printf("mean area = %f\n", sumArea / (double) pgs->n_zhou_c);
+      }
+
+      //    PMI(cov);
+      // PMI(pts);
+      
+      if (false)
+      printf("//cmaxDmu %4.1f; mass=%4.1f m1=%4.1e beta=%4.1f (%4.6f, %4.6f) (%4.6f, %4.6f) %4.3f %e\n",	     
+	     cmaxDmu, (double) pgs->totalmass, shape->mpp.mMplus[1], 
+	     PARAM0(shape->sub[0], 0), 
+	     shape->sub[0]->Spolygon->P->box0[0],
+	     shape->sub[0]->Spolygon->P->box0[1],
+	     shape->sub[0]->Spolygon->P->box1[0],
+	     shape->sub[0]->Spolygon->P->box1[1],
+	     101 * 101 *  shape->sub[0]->Spolygon->P->box1[0]* shape->sub[0]->Spolygon->P->box1[1] * 4,
+	     101 * 101 *  shape->sub[0]->Spolygon->P->box1[0]* shape->sub[0]->Spolygon->P->box1[1] * 4 -  pgs->totalmass);
+
+      //assert( shape->sub[0] != NULL);
+      //assert(shape->sub[0]->Spolygon != NULL);
+      //assert(shape->sub[0]	->Spolygon->P != NULL); 
 
 
-  if (!cov->deterministic) {
-    int err;
-    DO(shape, S); 
-    if ((err = INIT(pts, 1, S)) != NOERROR) {
-      XERR(err); 
-    }
-    DORANDOM(pts, cov->q);  // cov->q nur dummy. Wird ueberschrieben
-     // CovList[shape->nr].Do(shape, S); // nicht gatternr
-    if (calculate_mass_maxstable(cov) != NOERROR) 
-      error("unexpected error in 'do_pts_given_shape' (maxstable)");
+      //assert(false);
+      //    if (!(shape->mpp.mMplus[1] > 0.0001 || pgs->totalmass < 100)) APMI(cov);
+      //    assert( pgs->totalmass <= 101 * 101 *  shape->sub[0]->Spolygon->P->box1[0]* shape->sub[0]->Spolygon->P->box1[1] * 4 + 1e-12);
+
+      //PMI(cov);
+      //assert( shape->sub[0]->Spolygon->P->box0[0] == PARAM(pts, 0)[0]);
+
+      double old_zhou = pgs->cov->Spgs->old_zhou;
+      if (old_zhou < cmaxDmu || UNIFORM_RANDOM * old_zhou < cmaxDmu) {
+	//printf(". %f %f\n", old_zhou, cmaxDmu);
+	pgs->old_zhou = cmaxDmu;
+	SWAP_PGS;
+      } //else printf("* %f %f\n", old_zhou, cmaxDmu);
+    } // mcmc
+    
+    SWAP_PGS;
+    cov_model *prev=cov->calling;		
+    assert(prev != NULL);				
+    if (prev->key != NULL) prev->key = cov;		
+    else if (prev->sub[0] != NULL) prev->sub[0] = cov;	
+    else if (prev->sub[1] != NULL) prev->sub[1] = cov;	
+    else error("structure mismatch");			
+
+    // APMI(cov);
+
+
+    // printf("\n");
+    //PMI(pts);
+    // printf("%f %f\n", PARAM(pts, 1)[0] *PARAM(pts, 1)[1] * 4 , pts->mpp.mMplus[0]);
+    
+    //assert(fabs(PARAM(pts, 1)[0] *PARAM(pts, 1)[1] * 4 - pts->mpp.mMplus[0]) < 1e-14 *  pts->mpp.mMplus[0]);
+
+    //   PMI(save_pts);
+    //APMI(pts);
+
+    DORANDOM(pts, cov->q); // cov->q ist hier nur dummy    
+
+    // if (!newshape) printf("failed\n");
+    //printf("old %f %f %f\n", (double) pgs->old_zhou, (double) pgs->totalmass, PARAM0(shape->sub[0], POWSCALE));
+
+    // assert(pgs->old_zhou == pgs->totalmass);
+    // if (pgs->old_zhou > 1e6 && PARAM0(shape->sub[0], POWSCALE)>1) APMI(cov);  //    assert(shape->sub[0]->nr == POWER_DOLLAR && pts->nr == LOC);
+    // assert(PARAM0(shape->sub[0], POWSCALE) == PARAM0(pts, LOC_SCALE));
+
+
+    //
+    if (false)
+      printf("//zhou %4.1f %4.3f %ld mean=%4.3f, shape=%s %f \n", (double) pgs->sum_zhou_c, pgs->totalmass, pgs->n_zhou_c, 
+	     (double) pgs->sum_zhou_c / (double) pgs->n_zhou_c,
+	     NICK(shape->sub[0]), PARAM0(shape->sub[0], POWSCALE)
+	     ); // assert(pgs->n_zhou_c <= 500);
   }
 
   i = DrawCathegory(pgs->size, single, total, loc->grid, &elmts);
-
-  //   pgs:single 0.000000 1.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.020262 
-  //     pgs:total  0.000000 1.000000 1.000000 1.000000 1.000000 1.000000 1.000000 21.261911 
-   //  printf("i=%d total=%f grid=%d elm=%d sin=%f, size=%d\n", i, *total, loc->grid, elmts, *single, pgs->size);
  
   if (flat) for (d=0; d<dim; d++) x[d] = 0.0; 
   else for (d=0; d<dim; d++) x[d] = halfstepvector[d];
@@ -820,23 +1020,49 @@ void do_pgs_maxstable(cov_model *cov, storage *S) {
   } else BUG;
  
   //APMI(cov);
-
+  //printf("flat = %d %ld %s i=%d %d \n", flat, x, NICK(pts), i, PGS_VOXEL);
+ 
   if (flat) { 
+    // to do: programmierfehler!
+    // frage: lohnt sich 'flat=TRUE' ueberhaupt noch hinsichtlich Aufwand
+    //        und Rechenzeitersparnis?
+    // Bsp das nicht funktioniert fuer flat=FLAT_UNDETERMINED
+    //    x <- seq(1, 10, 0.01)
+    //      z <- RFsimulate(RPsmith(RMgauss(), xi=0), x, x, grid=T, print=20)
+    //      Print(exp(-exp(range(-z[[1]]))))
+    //      plot(z); X11(); hist(z[[1]], freq=FALSE, 50)
+    //			curve(exp(-x-2) * exp(-exp(-x-2)), -5, 2, add=TRUE)
     if (i != PGS_VOXEL) VTLG_R(x, pts, v); 
     //  printf("v=%f x=%f %d %d %d \n", v[0],  x[0], i, PGS_VOXEL,  PGS_CORNER);
     for (d = 0; d<dim; d++)
       if (x[d] == 0.0) 
 	v[d] = loc->xgr[d][XSTEP] * UNIFORM_RANDOM - halfstepvector[d];
   } else { 
-    //  printf("here\n");
+    //    printf("here %f %f %f\n", x[0], x[1], x[2]);
+    // double xx[3]; /* print */
+    //  xx[0] = xx[1] = 0.5;   xx[2] = RF_INF;  bad
+    // xx[0] = 0.5;  xx[1] = xx[2] = RF_INF; badd
+    // xx[0] = xx[1] = xx[2] = RF_INF; ok
+    //xx[0] = 0.5;  xx[1] = xx[2] = RF_INF;
     VTLG_R2SIDED(NULL, x, pts, v); 
   }
 
-  //printf("i=%d %f %f %f v=%f %f %f\n", i, x[0], x[1], x[2], v[0], v[1], v[2]);
+  // v[0] = (2 * UNIFORM_RANDOM - 1) * PARAM0(shape->sub[0], POWSCALE) ;
+  //  v[1] = (2 * UNIFORM_RANDOM - 1) * PARAM0(shape->sub[0], POWSCALE);
+  // v[2] = (2 * UNIFORM_RANDOM - 1) * PARAM0(shape->sub[0], POWSCALE);
+
+  //  if (v[0] * v[0] + v[1] * v[1] +  v[2] *  v[2] > 0.1001 * 0.1001 * 2)
+  // printf("i=%d %f %f %f v=%f %f %f\n", i, x[0], x[1], x[2], v[0], v[1], v[2]);
+
+  // APMI(cov);
+ 
+  //printf("i=%d %f %f v=%f %f\n", i, x[0], x[1], v[0], v[1]);
 
   for (d=0; d<dim; d++) {
     cov->q[d] = loc->xgr[d][XSTART] + v[d];
+    // 
     // printf("q=%f %f %f %d\n", cov->q[d], loc->xgr[d][XSTART], v[d], flat);
+//APMI(pts->calling);
     if (R_FINITE(x[d])) {      
       int 
 	len = (int) loc->xgr[d][XLENGTH] - 1,
@@ -861,52 +1087,60 @@ void do_pgs_maxstable(cov_model *cov, storage *S) {
     //   printf("D=%d %f %f %d\n", d, cov->q[d], v[d], elmts);
 
   }
-
-
-  //   if (counter[0]==0 && counter[1]==0) PMI(cov);
  
   if (flat) for (d = 0; d<dim; d++) if (x[d] == 0.0)  v[d] = 0.0;
  
-  if (CovList[pts->nr].logD != ErrLogCov) {
+  if (CovList[pts->nr].logD != ErrCov) {
     // printf("log!\n");
-    double sign;
-    VTLG_DLOG(v, pts, &(pgs->log_density), &sign); // hier muss allgemein die nicht-normierte Dichte stehen und stattdessen durch das Mittel des Volumens geteilt  werden
+    VTLG_DLOG(v, pts, &(pgs->log_density)); // hier muss allgemein die nicht-normierte Dichte stehen und stattdessen durch das Mittel des Volumens geteilt  werden
    
 
     
-    //      printf("log density v=%f, %f, %f %e\n", v[0],v[1],v[2], pgs->log_density);
+    //printf("log density v=%f, %f, %f %e\n", v[0],v[1],v[2], pgs->log_density);
 
-     //printf("log_dens %f\n", pgs->log_density);
+    //printf("log_dens %f\n", pgs->log_density);
     //  pgs->log_density -= log(pgs->totalmass); // ueberprueft: normierung
     // zu einer WK'Dichte.
   } else {
     double density;
     VTLG_D(v, pts, &density);
 
-    //    printf("density %e v=%f %f %f\n", density, v[0], v[1], v[2]);
+    //  printf("dens %f\n", density);
+      //  printf("density %e v=%f %f %f\n", density, v[0], v[1], v[2]);
 
     pgs->log_density = log(density);
   }
 
-  //counter[cov->q[1] > 0.5]++; 
- //printf("counter %d %d cat =%d v>0:%d q=(%f %f %f)\n", 
- //	counter[0], counter[1], i, v[1]>0, cov->q[0], cov->q[1], cov->q[2]
- //	);
- //assert(counter[0] < 40);
-
-  //  PMI(pts);
-  
+ 
   if (!R_FINITE(pgs->log_density)) {
     //PMI(cov);
     BUG;
   }
+
+  //assert(pgs->log_density == 0);
+
+  //  PMI(cov);
   
 }
 
 
 
-
 void do_pts_given_shape(cov_model *cov, storage *S) {
+  // muss zu allerst stehen, da cov sich aendern kann!
+  if (cov->role == ROLE_POISSON_GAUSS) {
+    do_pgs_gauss(cov, S);
+  } else if (hasMaxStableRole(cov)) { // todo: sauber trennen, wann 
+    do_pgs_maxstable(cov, S); 
+  } else {
+    PMI(cov); BUG; //
+  }
+  cov_model *prev=cov->calling;		
+  assert(prev != NULL);				
+  if (prev->key != NULL) cov = prev->key;		
+  else if (prev->sub[0] != NULL) cov = prev->sub[0];	
+  else if (prev->sub[1] != NULL) cov = prev->sub[1];	
+  else error("structure mismatch");			
+  
   cov_model *shape = cov->sub[PGS_FCT],
     *pts = cov->sub[PGS_LOC];
   pgs_storage *pgs = cov->Spgs;
@@ -918,55 +1152,79 @@ void do_pts_given_shape(cov_model *cov, storage *S) {
     *y = pgs->y;
 
   if (cov->role == ROLE_POISSON_GAUSS) {
-    do_pgs_gauss(cov, S);
     eps = GLOBAL.mpp.about_zero * exp(pgs->log_density);
   } else if (hasMaxStableRole(cov)) { // todo: sauber trennen, wann 
     // max-stable, smith etc
-    do_pgs_maxstable(cov, S); 
     eps = pgs->currentthreshold;
+
+    //printf("eps= %f\n", eps);
+
+    if (!R_FINITE(pgs->log_density)) {
+      //PMI(cov->Spgs->cov);   PMI(cov);
+      BUG;
+    }
     if (cov->loggiven)  eps += pgs->log_density;
     else eps *= exp(pgs->log_density);
-  } else {
-    PMI(cov); BUG; //
+  } else BUG;
+
+  
+  //printf("here %f %d\n", eps, cov->loggiven);
+  // APMI(cov);
+   
+  if (cov->loggiven) { NONSTATLOGINVERSE(&eps, shape, x, y); }
+  else NONSTATINVERSE(&eps, shape, x, y);
+
+  // assert(false);
+  // warum fkt obiges nicht bei gauss??
+  //  PMI(shape); 
+  //printf("eps=%f x=%f %d %f %f\n", eps, *x, cov->loggiven,
+  //
+  //	 pgs->currentthreshold, pgs->log_density);
+
+
+  if (ISNAN(x[0]) || x[0] > y[0]) {
+    //double eps_neu = eps / cov->mpp.maxheights[0]; // warum ?? 29.12.2013
+    double eps_neu = eps; //  29.12.2013
+    if (cov->loggiven) { BUG; }
+    else NONSTATINVERSE_D(&eps_neu, pts, x, y); 
+    
+    if (ISNAN(x[0])  || x[0] > y[0]) BUG;
+ 
+    //printf("eps=%f %f %f x=(%e %e %e) y=(%e %e %e) q=(%f %f %f)\n",
+    //     eps, pgs->currentthreshold,  exp(pgs->log_density),
+    //     x[0], x[1], x[2], y[0], y[1], y[2],
+    //     cov->q[0], cov->q[1], cov->q[2]);
   }
 
-  //  APMI(shape);
-  
-  NONSTATINVERSE(&eps, shape, x, y);
-  if (ISNA(x[0]) || x[0] > y[0]) {
-    assert(!cov->loggiven);
-    //double eps_neu = eps / cov->mpp.maxheight; // warum ?? 29.12.2013
-    double eps_neu = eps; //  29.12.2013
-    NONSTATINVERSE_D(&eps_neu, pts, x, y); 
-    if (ISNA(x[0]) || x[0] > y[0]) BUG;
- 
-      //printf("eps=%f %f %f x=(%e %e %e) y=(%e %e %e) q=(%f %f %f)\n",
-      //     eps, pgs->currentthreshold,  exp(pgs->log_density),
-      //     x[0], x[1], x[2], y[0], y[1], y[2],
-      //     cov->q[0], cov->q[1], cov->q[2]);
-  }
- 
-  //  assert(!R_FINITE(y[0]) || y[0] < 55.281);
+  //  printf("Xx=%f %d %d %f %f\n", *x, ISNAN(x[0]),cov->loggiven, pgs->currentthreshold, pgs->log_density);
+
 
   //APMI(cov);
   
   for (d=0; d<dim; d++) {
+ 
+    pgs->supportmin[d] = cov->q[d] - y[d]; // 4 * for debugging...
+    pgs->supportmax[d] = cov->q[d] - x[d];
 
-    //     printf("do d=%d q=%f x=%f y=%f eps=%e thr=%e\n",
-    //	  d, cov->q[d], x[d], y[d], eps, pgs->currentthreshold);
+     //     printf("do d=%d q=%f x=%f y=%f eps=%e thr=%e %f %f\n",
+    //	  	  d, cov->q[d], x[d], y[d], eps, pgs->currentthreshold, pgs->supportmin[d], pgs->supportmax[d]);
 
-    pgs->supportmin[d] = cov->q[d] - 10 * y[d]; // 4 * for debugging...
-    pgs->supportmax[d] = cov->q[d] - 10 * x[d];
-
-    if (ISNA(pgs->supportmin[d]) || ISNA(pgs->supportmax[d]) ||
+  if (ISNAN(pgs->supportmin[d]) || ISNAN(pgs->supportmax[d]) ||
 	pgs->supportmin[d] > pgs->supportmax[d]) {
-      //printf("do d=%d q=%f x=%f y=%f eps=%e thr=%e supp=(%e, %e)\n",    d, cov->q[d], x[d], y[d], eps, pgs->currentthreshold,  pgs->supportmin[d], pgs->supportmax[d]);
+      //      printf("do d=%d q=%f min=%f max=%f eps=%e thr=%e supp=(%e, %e)\n",    d, cov->q[d], x[d], y[d], eps, pgs->currentthreshold,  pgs->supportmin[d], pgs->supportmax[d]);
+      // APMI(shape);
 
-      BUG;
+// BUG;
     }
   
     //assert(pgs->supportmin[d] <= pgs->supportmax[d]);
   }
+  // assert(false);
+  
+
+  //  BUG;
+
+  //   APMI(cov);
 
   cov->fieldreturn = shape->fieldreturn;
   cov->origrf = false;
@@ -987,6 +1245,27 @@ void range_pts_given_shape(cov_model VARIABLE_IS_NOT_USED *cov, range_type *rang
   range->pmax[PGS_FLAT] = 1;
   range->openmin[PGS_FLAT] = false;
   range->openmax[PGS_FLAT] = false; 
+
+  range->min[PGS_INFTY_SMALL] = 0;
+  range->max[PGS_INFTY_SMALL] = 1;
+  range->pmin[PGS_INFTY_SMALL] = 0;
+  range->pmax[PGS_INFTY_SMALL] = 1;
+  range->openmin[PGS_INFTY_SMALL] = false;
+  range->openmax[PGS_INFTY_SMALL] = false; 
+
+  range->min[PGS_NORMED] = 0;
+  range->max[PGS_NORMED] = 1;
+  range->pmin[PGS_NORMED] = 0;
+  range->pmax[PGS_NORMED] = 1;
+  range->openmin[PGS_NORMED] = false;
+  range->openmax[PGS_NORMED] = false; 
+
+  range->min[PGS_ISOTROPIC] = 0;
+  range->max[PGS_ISOTROPIC] = 1;
+  range->pmin[PGS_ISOTROPIC] = 0;
+  range->pmax[PGS_ISOTROPIC] = 1;
+  range->openmin[PGS_ISOTROPIC] = false;
+  range->openmax[PGS_ISOTROPIC] = false; 
 }
 
 
@@ -1029,6 +1308,8 @@ int check_standard_shape(cov_model *cov) {
 		     SCALAR, role)) != NOERROR)  return err;
   setbackward(cov, shape);
   
+  if (!shape->deterministic) SERR("random shapes for 'standard' not allowed yet");
+
   if (pts != NULL) {
     if ((err = CHECK_R(pts, dim)) != NOERROR) return err;
   }
@@ -1070,18 +1351,21 @@ int init_standard_shape(cov_model *cov, storage *S) {
   int d,
     dim = shape->xdimprev,
     err = NOERROR;
- 
+  pgs_storage *pgs = cov->Spgs;
+
   //APMI(cov);
 
   assert(shape->xdimprev == Loc(cov)->timespacedim);
 
-  if ((err = alloc_pgs(cov)) != NOERROR) return err;
-  pgs_storage *pgs = cov->Spgs;
-  if ((pgs->localmin = (double*) CALLOC(dim, sizeof(double))) == NULL ||
-      (pgs->localmax = (double*) CALLOC(dim, sizeof(double))) == NULL ||
-      (pgs->minmean = (double*) CALLOC(dim, sizeof(double))) == NULL ||
-      (pgs->maxmean = (double*) CALLOC(dim, sizeof(double))) == NULL)
-    return ERRORMEMORYALLOCATION;
+  if (pgs == NULL) {
+    if ((err = alloc_pgs(cov)) != NOERROR) return err;
+    pgs = cov->Spgs;
+    if ((pgs->localmin = (double*) CALLOC(dim, sizeof(double))) == NULL ||
+	(pgs->localmax = (double*) CALLOC(dim, sizeof(double))) == NULL ||
+	(pgs->minmean = (double*) CALLOC(dim, sizeof(double))) == NULL ||
+	(pgs->maxmean = (double*) CALLOC(dim, sizeof(double))) == NULL)
+      return ERRORMEMORYALLOCATION;
+  }
 
  
   // selbst wenn zufaelliger Shape: 1x laufen lassen, ob 
@@ -1105,7 +1389,7 @@ int init_standard_shape(cov_model *cov, storage *S) {
   
   // try out whether it works:
   NONSTATINVERSE(ZERO, shape, x, y);
-  if (ISNA(x[0]) || x[0] > y[0])
+  if (ISNAN(x[0]) || x[0] > y[0])
     SERR1("inverse of '%s' unknown", NICK(shape));
   GetDiameter(loc, pgs->localmin, pgs->localmax, 
 	      pgs->supportcentre); // last is only a dummy
@@ -1122,14 +1406,17 @@ int init_standard_shape(cov_model *cov, storage *S) {
     pgs->log_density = 0.0;
   } else {    
     pgs->log_density = 0.0;
-    cov->mpp.log_zhou_c = log(pgs->totalmass); // todo: logzhou gehoert in pgs
-    cov->mpp.maxheight = shape->mpp.maxheight;
+    pgs->zhou_c = pgs->totalmass / shape->mpp.mMplus[1];
+    cov->mpp.maxheights[0] = shape->mpp.maxheights[0];
+    pgs->estimated_zhou_c = !cov->deterministic;
+    if (pgs->estimated_zhou_c) SERR("random shapes in standard approach not coded yet -- please contact author");
   }
 
+
   cov->rf = shape->rf;
   cov->origrf = false;
   cov->fieldreturn = shape->fieldreturn;
-
+ 
   // APMI(cov);
 
   return NOERROR;
@@ -1158,7 +1445,7 @@ void do_standard_shape(cov_model *cov, storage *S) {
 
   //PMI(shape, "standard");
   NONSTATINVERSE(ZERO, shape, x, y);
-  if (ISNA(x[0])|| x[0] > y[0]) BUG;
+  if (ISNAN(x[0])|| x[0] > y[0]) BUG;
 
   for (d=0; d<dim; d++) {
 
@@ -1250,7 +1537,7 @@ int struct_stationary_shape(cov_model *cov, cov_model **newmodel){
 
 int init_stationary_shape(cov_model *cov, storage *S) {  
   cov_model *shape = cov->sub[STAT_SHAPE_FCT];
-   int d,
+  int d, i,
     err = NOERROR,
     dim = shape->xdimprev;
  
@@ -1267,12 +1554,18 @@ int init_stationary_shape(cov_model *cov, storage *S) {
   
   assert(cov->mpp.moments >= 1);
   if ((err = INIT(shape, 1, S)) != NOERROR) return err; //gatter?
-
   assert(shape->mpp.moments >= 1);
- 
-  cov->mpp.log_zhou_c = 0.0;
-  if (!R_FINITE(cov->mpp.log_zhou_c))
+  for (i=0; i<=cov->mpp.moments; i++) {
+    cov->mpp.mM[i] = shape->mpp.mM[i];
+    cov->mpp.mMplus[i] = shape->mpp.mMplus[i];
+  }
+
+   
+  pgs->zhou_c = 1.0 / cov->mpp.mMplus[1]; // passt fuer binary, und auch fuer 
+  if (!R_FINITE(pgs->zhou_c))
     SERR1("max height of '%s' not finite", NICK(shape));
+  pgs->estimated_zhou_c = false; 
+  if (!cov->deterministic) SERR("not deterministic shapes in stationary modelling -- please contact author");
   
   pgs->log_density = 0; 
    
@@ -1283,12 +1576,15 @@ int init_stationary_shape(cov_model *cov, storage *S) {
     pgs->supportmax[d] = RF_INF;
   }
 
-  cov->mpp.maxheight = shape->mpp.maxheight;
+  cov->mpp.maxheights[0] = shape->mpp.maxheights[0];
   cov->rf = shape->rf;
   cov->origrf = false;
   cov->fieldreturn = shape->fieldreturn;
   if (!cov->fieldreturn) BUG;
 
+
+  // APMI(cov);
+
   return NOERROR;
 }
 
@@ -1296,7 +1592,7 @@ int init_stationary_shape(cov_model *cov, storage *S) {
 void do_stationary_shape(cov_model *cov, storage *S) {
   cov_model *shape = cov->sub[STAT_SHAPE_FCT]; 
   DO(shape, S);
-  cov->mpp.maxheight = shape->mpp.maxheight;
+  cov->mpp.maxheights[0] = shape->mpp.maxheights[0];
   assert(shape->fieldreturn);
 }
 
